@@ -10,6 +10,11 @@ citation_format    Format for one MedlineCitation.
 format             Format for a whole file.
 
 """
+
+import warnings
+warnings.warn("Bio.Medline.NLMMedlineXML was deprecated, as it does not seem to be able to parse recent Medline XML files. If you want to continue to use this module, please get in contact with the Biopython developers at biopython-dev@biopython.org to avoid permanent removal of this module from Biopython", DeprecationWarning)
+
+
 import sys
 
 from Martel import *
