@@ -31,7 +31,7 @@ else:
 
 INTERLEAVE=70
 SPECIAL_COMMANDS=['charstatelabels','charlabels','taxlabels', 'taxset', 'charset','charpartition','taxpartition',\
-        'matrix','tree','translate']
+        'matrix','tree', 'utree','translate']
 KNOWN_NEXUS_BLOCKS = ['trees','data', 'characters', 'taxa', 'sets']
 PUNCTUATION='()[]{}/\,;:=*\'"`+-<>'
 MRBAYESSAFE='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_'
@@ -230,7 +230,10 @@ def get_start_end(sequence, skiplist=['-','?']):
     start=0
     while start<length and (sequence[start] in skiplist):
         start+=1
-    return start,end 
+    if start==length and end==-1: # empty sequence
+        return -1,-1
+    else:
+        return start,end 
     
 def _sort_keys_by_values(p):
     """Returns a sorted list of keys of p sorted by values of p."""     
@@ -548,13 +551,16 @@ class Nexus(object):
             #3 Assume we have a file object
             elif hasattr(input,'read'): # file objects or StringIO objects
                 file_contents=input.read()
-                if input.name:
+                if hasattr(input,"name") and input.name:
                     self.filename=input.name
                 else:
                     self.filename='Unknown_nexus_file'
             else:
-                print input.strip()[:6]
+                print input.strip()[:50]
                 raise NexusError, 'Unrecognized input: %s ...' % input[:100]
+        file_contents=file_contents.strip()
+        if file_contents.startswith('#NEXUS'):
+            file_contents=file_contents[6:]
         if C:
             decommented=cnexus.scanfile(file_contents)
             #check for unmatched parentheses
@@ -840,7 +846,7 @@ class Nexus(object):
             if block_interleave==0:
                 while self.matrix.has_key(id):
                     if id.split('.')[-1].startswith('copy'):
-                        id='.'.join(id.split('.')[:-1])+'.copy'+str(eval(id.split('.')[-1][4:])+1)
+                        id='.'.join(id.split('.')[:-1])+'.copy'+str(eval('0'+id.split('.')[-1][4:])+1)
                     else:
                         id+='.copy'
                     #raise NexusError, id+' already in matrix!\nError in: '+l
@@ -879,6 +885,10 @@ class Nexus(object):
             except:
                 raise NexusError,'Format error in line %s.' % options
 
+    def _utree(self,options):
+        """Some software (clustalx) uses 'utree' to denote an unrooted tree."""
+        self._tree(options)
+        
     def _tree(self,options):
         opts=CharBuffer(options)
         name=opts.next_word()
@@ -1163,7 +1173,7 @@ class Nexus(object):
         if not filename:
             filename=self.filename
         if [t for t in delete if not self._check_taxlabels(t)]:
-            raise NexusError, 'Unknwon taxa: %s' % ', '.join(sets.Set(delete).difference(sets.Set(self.taxlabels)))
+            raise NexusError, 'Unknown taxa: %s' % ', '.join(sets.Set(delete).difference(sets.Set(self.taxlabels)))
         if interleave_by_partition:
             if not interleave_by_partition in self.charpartitions:
                 raise NexusError, 'Unknown partition: '+interleave_by_partition
@@ -1566,8 +1576,11 @@ class Nexus(object):
             sequence=self.matrix[taxon].tostring()
             length=len(sequence)
             start,end=get_start_end(sequence,skiplist=replace)
-            sequence=sequence[:end+1]+missing*(length-end-1)
-            sequence=start*missing+sequence[start:]
+            if start==-1 and end==-1:
+                sequence=missing*length
+            else:
+                sequence=sequence[:end+1]+missing*(length-end-1)
+                sequence=start*missing+sequence[start:]
             assert length==len(sequence), 'Illegal sequence manipulation in Nexus.termial_gap_to_missing in taxon %s' % taxon
             self.matrix[taxon]=Seq(sequence,self.alphabet)
 
