@@ -9,48 +9,13 @@
 
 import os, sys, getopt, re, time
 from string import maketrans
-from Bio import Fasta
+from Bio import SeqIO
 from Bio import Translate
 from Bio.Seq import Seq
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
 from Bio.Data import IUPACData, CodonTable
 
-### The following section was replaced by the functions complement,
-### forward_complement in Bio.Seq. The code here can be removed eventually
-### (using deprecation warnings for now).
-
-_before = ''.join(IUPACData.ambiguous_dna_complement.keys())
-_after = ''.join(IUPACData.ambiguous_dna_complement.values())
-_ttable = maketrans(_before, _after)
-
-def complement(seq):
-    """Returns the complementary sequence (NOT antiparallel).
-
-    This works on string sequences, not on Bio.Seq objects.
-    """
-    #Much faster on really long sequences than the previous loop based one.
-    #thx to Michael Palmer, University of Waterloo
-    import warnings
-    warnings.warn(
-        "complement in Bio.SeqUtils is deprecated; please use complement in Bio.Seq instead",
-        category=DeprecationWarning)
-    return seq.translate(_ttable)
-
-def antiparallel(seq):
-   """returns reversed complementary sequence ( = other strand )
-
-   This works on string sequences, not on Bio.Seq objects.
-   """
-   import warnings
-   warnings.warn(
-       "antiparallel in Bio.SeqUtils is deprecated; please use reverse_complement in Bio.Seq instead",
-       category=DeprecationWarning)
-   s = complement(seq)
-   s = reverse(s)
-   return s
-
-### End section to be removed.
 
 ######################################
 # DNA
@@ -81,19 +46,19 @@ def GC(seq):
    return gc*100.0/len(seq)
     
 def GC123(seq):
-   " calculates totla G+C content plus first, second and third position "
-   l = len(seq)
+   " calculates total G+C content plus first, second and third position "
 
    d= {}
    for nt in ['A','T','G','C']:
       d[nt] = [0,0,0]
 
-   for i in range(0,l,3):
+   for i in range(0,len(seq),3):
       codon = seq[i:i+3]
       if len(codon) <3: codon += '  '
       for pos in range(0,3):
          for nt in ['A','T','G','C']:
-            if codon[pos] == nt: d[nt][pos] = d[nt][pos] +1
+            if codon[pos] == nt or codon[pos] == nt.lower():
+                d[nt][pos] = d[nt][pos] +1
 
 
    gc = {}
@@ -398,13 +363,10 @@ def apply_on_multi_fasta(file, function, *args):
    except:
       raise NotImplementedError, "%s not implemented" % function
    
-   parser = Fasta.RecordParser()
    handle = open(file, 'r')
-   iter = Fasta.Iterator(handle, parser)
+   records = SeqIO.parse(handle, "fasta")
    results = []
-   while 1:
-      record = iter.next()
-      if not record: break
+   for record in records:
       arguments = [record.sequence]
       for arg in args: arguments.append(arg)
       result = f(*arguments)
