@@ -11,6 +11,9 @@ http://www.expasy.ch/sprot/sprot-top.html
 Tested with:
 Release 37, Release 38, Release 39
 
+Limited testing with:
+Release 51, 54
+
 
 Classes:
 Record             Holds SwissProt data.
@@ -41,6 +44,8 @@ from Bio.ParserSupport import *
 from Bio.WWW import ExPASy
 from Bio.WWW import RequestLimiter
 
+_CHOMP = " \n\r\t.,;" #whitespace and trailing punctuation
+
 class Record:
     """Holds information from a SwissProt record.
 
@@ -62,6 +67,8 @@ class Record:
     organism_classification  The taxonomy classification.  List of strings.
                              (http://www.ncbi.nlm.nih.gov/Taxonomy/)
     taxonomy_id       A list of NCBI taxonomy id's.
+    host_organism     A list of NCBI taxonomy id's of the hosts of a virus,
+                      if any.
     references        List of Reference objects.
     comments          List of strings.
     cross_references  List of tuples (db, id1[, id2][, id3]).  See the docs.
@@ -91,6 +98,7 @@ class Record:
         self.organelle = ''
         self.organism_classification = []
         self.taxonomy_id = []
+        self.host_organism = []
         self.references = []
         self.comments = []
         self.cross_references = []
@@ -331,9 +339,23 @@ class _Scanner:
         while uhandle.peekline():
             self._scan_record(uhandle, consumer)
 
+    def _skip_starstar(self, uhandle) :
+        """Ignores any lines starting **"""
+        #See Bug 2353, some files from the EBI have extra lines
+        #starting "**" (two asterisks/stars), usually between the
+        #features and sequence but not all the time.  They appear
+        #to be unofficial automated annotations. e.g.
+        #**
+        #**   #################    INTERNAL SECTION    ##################
+        #**HA SAM; Annotated by PicoHamap 1.88; MF_01138.1; 09-NOV-2003.
+        while "**" == uhandle.peekline()[:2] :
+            skip = uhandle.readline()
+            #print "Skipping line: %s" % skip.rstrip()
+
     def _scan_record(self, uhandle, consumer):
         consumer.start_record()
         for fn in self._scan_fns:
+            self._skip_starstar(uhandle)
             fn(self, uhandle, consumer)
 
             # In Release 38, ID N33_HUMAN has a DR buried within comments.
@@ -402,6 +424,10 @@ class _Scanner:
         self._scan_line('OX', uhandle, consumer.taxonomy_id,
                         any_number=1)
 
+    def _scan_oh(self, uhandle, consumer):
+        # viral host organism. introduced after SwissProt 39.
+        self._scan_line('OH', uhandle, consumer.organism_host, any_number=1)
+
     def _scan_reference(self, uhandle, consumer):
         while 1:
             if safe_peekline(uhandle)[:2] != 'RN':
@@ -413,6 +439,10 @@ class _Scanner:
             # ws:2001-12-05 added, for record with RL before RA
             self._scan_rl(uhandle, consumer)
             self._scan_ra(uhandle, consumer)
+            #EBI copy of P72010 is missing the RT line, and has one
+            #of their ** lines in its place noting "**   /NO TITLE."
+            #See also bug 2353
+            self._skip_starstar(uhandle) 
             self._scan_rt(uhandle, consumer)
             self._scan_rl(uhandle, consumer)
     
@@ -468,7 +498,10 @@ class _Scanner:
     
     def _scan_ft(self, uhandle, consumer):
         self._scan_line('FT', uhandle, consumer.feature_table, any_number=1)
-    
+
+    def _scan_pe(self, uhandle, consumer):
+        self._scan_line('PE', uhandle, consumer.protein_existence, any_number=1)
+
     def _scan_sq(self, uhandle, consumer):
         self._scan_line('SQ', uhandle, consumer.sequence_header, exactly_one=1)
     
@@ -488,9 +521,11 @@ class _Scanner:
         _scan_og,
         _scan_oc,
         _scan_ox,
+        _scan_oh,
         _scan_reference,
         _scan_cc,
         _scan_dr,
+        _scan_pe,
         _scan_kw,
         _scan_ft,
         _scan_sq,
@@ -508,6 +543,9 @@ class _RecordConsumer(AbstractConsumer):
     def __init__(self):
         self.data = None
         
+    def __repr__(self) :
+        return "Bio.SwissProt.SProt._RecordConsumer()"
+        
     def start_record(self):
         self.data = Record()
         
@@ -516,77 +554,152 @@ class _RecordConsumer(AbstractConsumer):
 
     def identification(self, line):
         cols = line.split()
-        self.data.entry_name = cols[1]
-        self.data.data_class = self._chomp(cols[2])    # don't want ';'
-        self.data.molecule_type = self._chomp(cols[3]) # don't want ';'
-        self.data.sequence_length = int(cols[4])
-
+        #Prior to release 51, included with MoleculeType:
+        #ID   EntryName DataClass; MoleculeType; SequenceLength.
+        #
+        #Newer files lack the MoleculeType:
+        #ID   EntryName DataClass; SequenceLength.
+        #
+        #Note that cols is split on white space, so the length
+        #should become two fields (number and units)
+        if len(cols) == 6 :
+            self.data.entry_name = cols[1]
+            self.data.data_class = cols[2].rstrip(_CHOMP)    # don't want ';'
+            self.data.molecule_type = cols[3].rstrip(_CHOMP) # don't want ';'
+            self.data.sequence_length = int(cols[4])
+        elif len(cols) == 5 :
+            self.data.entry_name = cols[1]
+            self.data.data_class = cols[2].rstrip(_CHOMP)    # don't want ';'
+            self.data.molecule_type = None
+            self.data.sequence_length = int(cols[3])
+        else :
+            #Should we print a warning an continue?
+            raise SyntaxError("ID line has unrecognised format:\n"+line)
+        
         # data class can be 'STANDARD' or 'PRELIMINARY'
         # ws:2001-12-05 added IPI
-        if self.data.data_class not in ['STANDARD', 'PRELIMINARY', 'IPI']: 
+        # pjc:2006-11-02 added 'Reviewed' and 'Unreviewed'
+        if self.data.data_class not in ['STANDARD', 'PRELIMINARY', 'IPI',
+                                        'Reviewed', 'Unreviewed']: 
             raise SyntaxError, "Unrecognized data class %s in line\n%s" % \
                   (self.data.data_class, line)
         # molecule_type should be 'PRT' for PRoTein
-        if self.data.molecule_type != 'PRT':
+        # Note that has been removed in recent releases (set to None)
+        if self.data.molecule_type is not None \
+        and self.data.molecule_type != 'PRT':
             raise SyntaxError, "Unrecognized molecule type %s in line\n%s" % \
                   (self.data.molecule_type, line)
     
     def accession(self, line):
-        cols = self._chomp(line[5:].rstrip()).split(';')
+        cols = line[5:].rstrip(_CHOMP).split(';')
         for ac in cols:
             self.data.accessions.append(ac.lstrip())
     
     def date(self, line):
         uprline = string.upper(line)
-        
-        # find where the version information will be located
-        # This is needed for when you have cases like IPI where
-        # the release verison is in a different spot:
-        # DT   08-JAN-2002 (IPI Human rel. 2.3, Created)
-        uprcols = uprline.split()
-        rel_index = -1
-        for index in range(len(uprcols)):
-            if uprcols[index].find("REL.") >= 0:
-                rel_index = index
-        assert rel_index >= 0, \
-                "Could not find Rel. in DT line: %s" % (line)
-        version_index = rel_index + 1
-        # get the version information
-        cols = line.split()
-        str_version = self._chomp(cols[version_index])
-        # no version number
-        if str_version == '':
-            version = 0
-        # dot versioned
-        elif str_version.find(".") >= 0:
-            version = str_version
-        # integer versioned
-        else:
-            version = int(str_version)
+        cols = line.rstrip().split()
+                
+        if uprline.find('CREATED') >= 0 \
+        or uprline.find('LAST SEQUENCE UPDATE') >= 0 \
+        or uprline.find('LAST ANNOTATION UPDATE') >= 0:
+            # Old style DT line
+            # =================
+            # e.g.
+            # DT   01-FEB-1995 (Rel. 31, Created)
+            # DT   01-FEB-1995 (Rel. 31, Last sequence update)
+            # DT   01-OCT-2000 (Rel. 40, Last annotation update)
+            #
+            # or:
+            # DT   08-JAN-2002 (IPI Human rel. 2.3, Created)
+            # ... 
+            
+            # find where the version information will be located
+            # This is needed for when you have cases like IPI where
+            # the release verison is in a different spot:
+            # DT   08-JAN-2002 (IPI Human rel. 2.3, Created)
+            uprcols = uprline.split()
+            rel_index = -1
+            for index in range(len(uprcols)):
+                if uprcols[index].find("REL.") >= 0:
+                    rel_index = index
+            assert rel_index >= 0, \
+                    "Could not find Rel. in DT line: %s" % (line)
+            version_index = rel_index + 1
+            # get the version information
+            str_version = cols[version_index].rstrip(_CHOMP)
+            # no version number
+            if str_version == '':
+                version = 0
+            # dot versioned
+            elif str_version.find(".") >= 0:
+                version = str_version
+            # integer versioned
+            else:
+                version = int(str_version)
 
-        if uprline.find("CREATED") >= 0:
-            self.data.created = cols[1], version
-        elif uprline.find('LAST SEQUENCE UPDATE') >= 0:
-            self.data.sequence_update = cols[1], version
-        elif uprline.find( 'LAST ANNOTATION UPDATE') >= 0:
-            self.data.annotation_update = cols[1], version
+            if uprline.find('CREATED') >= 0:
+                self.data.created = cols[1], version
+            elif uprline.find('LAST SEQUENCE UPDATE') >= 0:
+                self.data.sequence_update = cols[1], version
+            elif uprline.find( 'LAST ANNOTATION UPDATE') >= 0:
+                self.data.annotation_update = cols[1], version
+            else:
+                assert False, "Shouldn't reach this line!"
+        elif uprline.find('INTEGRATED INTO') >= 0 \
+        or uprline.find('SEQUENCE VERSION') >= 0 \
+        or uprline.find('ENTRY VERSION') >= 0:
+            # New style DT line
+            # =================
+            # As of UniProt Knowledgebase release 7.0 (including
+            # Swiss-Prot release 49.0 and TrEMBL release 32.0) the
+            # format of the DT lines and the version information
+            # in them was changed - the release number was dropped.
+            #
+            # For more information see bug 1948 and
+            # http://ca.expasy.org/sprot/relnotes/sp_news.html#rel7.0
+            #
+            # e.g.
+            # DT   01-JAN-1998, integrated into UniProtKB/Swiss-Prot.
+            # DT   15-OCT-2001, sequence version 3.
+            # DT   01-APR-2004, entry version 14.
+            #
+            #This is a new style DT line...
+
+            # The date should be in string cols[1]
+            # Get the version number if there is one.
+            # For the three DT lines above: 0, 3, 14
+            try:
+                version = int(cols[-1])
+            except ValueError :
+                version = 0
+
+            # Re-use the historical property names, even though
+            # the meaning has changed slighty:
+            if uprline.find("INTEGRATED") >= 0:
+                self.data.created = cols[1], version
+            elif uprline.find('SEQUENCE VERSION') >= 0:
+                self.data.sequence_update = cols[1], version
+            elif uprline.find( 'ENTRY VERSION') >= 0:
+                self.data.annotation_update = cols[1], version
+            else:
+                assert False, "Shouldn't reach this line!"
         else:
             raise SyntaxError, "I don't understand the date line %s" % line
     
     def description(self, line):
-        self.data.description = self.data.description + line[5:]
+        self.data.description += line[5:]
     
     def gene_name(self, line):
-        self.data.gene_name = self.data.gene_name + line[5:]
+        self.data.gene_name += line[5:]
     
     def organism_species(self, line):
-        self.data.organism = self.data.organism + line[5:]
+        self.data.organism += line[5:]
     
     def organelle(self, line):
-        self.data.organelle = self.data.organelle + line[5:]
+        self.data.organelle += line[5:]
     
     def organism_classification(self, line):
-        line = self._chomp(line[5:].rstrip())
+        line = line[5:].rstrip(_CHOMP)
         cols = line.split(';')
         for col in cols:
             self.data.organism_classification.append(col.lstrip())
@@ -603,7 +716,7 @@ class _RecordConsumer(AbstractConsumer):
         # To parse this, I need to check to see whether I'm at the
         # first line.  If I am, grab the description and make sure
         # it's an NCBI ID.  Then, grab all the id's.
-        line = self._chomp(line[5:].rstrip())
+        line = line[5:].rstrip(_CHOMP)
         index = line.find('=')
         if index >= 0:
             descr = line[:index]
@@ -612,7 +725,20 @@ class _RecordConsumer(AbstractConsumer):
         else:
             ids = line.split(',')
         self.data.taxonomy_id.extend([id.strip() for id in ids])
-    
+
+    def organism_host(self, line):
+        # Line type OH (Organism Host) for viral hosts
+        # same code as in taxonomy_id()
+        line = line[5:].rstrip(_CHOMP)
+        index = line.find('=')
+        if index >= 0:
+            descr = line[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = line[index+1:].split(',')
+        else:
+            ids = line.split(',')
+        self.data.host_organism.extend([id.strip() for id in ids])
+        
     def reference_number(self, line):
         rn = line[5:].rstrip()
         assert rn[0] == '[' and rn[-1] == ']', "Missing brackets %s" % rn
@@ -671,28 +797,25 @@ class _RecordConsumer(AbstractConsumer):
         # reported by edvard@farmasi.uit.no
         # and these can be more complicated like:
         # RX   MEDLINE=95385798; PubMed=7656980;
+        # RX   PubMed=15060122; DOI=10.1136/jmg 2003.012781;
         # We look for these cases first and deal with them
-        if line.find( "=") != -1:
-            cols = line.split()
-            assert len(cols) > 1, "I don't understand RX line %s" % line
-
-            for info_col in cols[1:]:
-                id_cols = info_col.split("=")
-                if len(id_cols) == 2:
-                    self.data.references[-1].references.append(
-                        (self._chomp(id_cols[0]), self._chomp(id_cols[1])))
-                else:
-                    raise AssertionError("I don't understand RX line %s"
-                                         % line)
+        if line.find("=") != -1:
+            cols = line[2:].split("; ")
+            cols = [x.strip() for x in cols]
+            cols = [x for x in cols if x]
+            for col in cols:
+                x = col.split("=")
+                assert len(x) == 2, "I don't understand RX line %s" % line
+                key, value = x[0].rstrip(_CHOMP), x[1].rstrip(_CHOMP)
+                ref = self.data.references[-1].references
+                ref.append((key, value))
         # otherwise we assume we have the type 'RX   MEDLINE; 85132727.'
         else:
             cols = line.split()
             # normally we split into the three parts
-            if len(cols) == 3:
-                self.data.references[-1].references.append(
-                    (self._chomp(cols[1]), self._chomp(cols[2])))
-            else:
-                raise AssertionError("I don't understand RX line %s" % line)
+            assert len(cols) == 3, "I don't understand RX line %s" % line
+            self.data.references[-1].references.append(
+                (cols[1].rstrip(_CHOMP), cols[2].rstrip(_CHOMP)))
     
     def reference_author(self, line):
         assert self.data.references, "RA: missing RN"
@@ -738,12 +861,12 @@ class _RecordConsumer(AbstractConsumer):
         i = line.find('[')
         if i >= 0:
             line = line[:i]
-        cols = self._chomp(line.rstrip()).split(';')
+        cols = line.rstrip(_CHOMP).split(';')
         cols = [col.lstrip() for col in cols]
         self.data.cross_references.append(tuple(cols))
     
     def keyword(self, line):
-        cols = self._chomp(line[5:].rstrip()).split(';')
+        cols = line[5:].rstrip(_CHOMP).split(';')
         self.data.keywords.extend([c.lstrip() for c in cols])
 
     def feature_table(self, line):
@@ -803,6 +926,10 @@ class _RecordConsumer(AbstractConsumer):
 
         return description
     
+    def protein_existence(self, line):
+        #TODO - Record this information?
+        pass
+    
     def sequence_header(self, line):
         cols = line.split()
         assert len(cols) == 8, "I don't understand SQ line %s" % line
@@ -816,14 +943,6 @@ class _RecordConsumer(AbstractConsumer):
     def terminator(self, line):
         pass
 
-    # from Python 2.2.2 could be replaced with word.rstrip(".,;")
-    # if there is always only one puctuation
-    def _chomp(self, word, to_chomp='.,;'):
-        # Remove the punctuation at the end of a word.
-        if word[-1] in to_chomp:
-            return word[:-1]
-        return word
-
     #def _clean(self, line, rstrip=1):
     #    if rstrip:
     #        return string.rstrip(line[5:])
@@ -852,6 +971,7 @@ class _SequenceConsumer(AbstractConsumer):
     data      Record with SwissProt data.
     alphabet  The alphabet the generated Seq objects will have.
     """
+    #TODO - Cope with references as done for GenBank
     def __init__(self, alphabet = Alphabet.generic_protein):
         """Initialize a Sequence Consumer
 
@@ -866,6 +986,7 @@ class _SequenceConsumer(AbstractConsumer):
         seq = Seq.Seq("", self.alphabet)
         self.data = SeqRecord.SeqRecord(seq)
         self.data.description = ""
+        self.data.name = ""
         
     def end_record(self):
         self.data.description = self.data.description.rstrip()
@@ -875,8 +996,18 @@ class _SequenceConsumer(AbstractConsumer):
         self.data.name = cols[1]
 
     def accession(self, line):
+        #Note that files can and often do contain multiple AC lines.
         ids = line[5:].rstrip().split(';')
-        self.data.id = ids[0]
+        
+        #Use the first as the ID, but record them ALL in the annotations
+        try :
+            self.data.annotations['accessions'].extend(ids)
+        except KeyError :
+            self.data.annotations['accessions'] = ids
+            
+        #Use the FIRST accession as the ID, not the first on this line!
+        self.data.id = self.data.annotations['accessions'][0]
+        #self.data.id = ids[0]
 
     def description(self, line):
         self.data.description = self.data.description + \
@@ -887,6 +1018,97 @@ class _SequenceConsumer(AbstractConsumer):
                       self.alphabet)
         self.data.seq = self.data.seq + seq
 
+    def gene_name(self, line):
+        #We already store the identification/accession as the records name/id
+        try :
+            self.data.annotations['gene_name'] += line[5:]
+        except KeyError :
+            self.data.annotations['gene_name'] =  line[5:]
+
+    def comment(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores the comments as a long string with newlines
+        #with key 'comment'
+        try :
+            self.data.annotations['comment'] += "\n" + line[5:]
+        except KeyError :
+            self.data.annotations['comment'] =  line[5:]
+        #TODO - Follow SwissProt conventions more closely?
+            
+    def date(self, line):
+        date_str = line.split()[0]
+        uprline = string.upper(line)
+        if uprline.find('CREATED') >= 0 :
+            #Try and agree with SeqRecord convention from the GenBank parser,
+            #which stores the submitted date as 'date'
+            self.data.annotations['date'] = date_str
+        elif uprline.find('LAST SEQUENCE UPDATE') >= 0 :
+            #There is no existing convention from the GenBank SeqRecord parser
+            self.data.annotations['date_last_sequence_update'] = date_str
+        elif uprline.find('LAST ANNOTATION UPDATE') >= 0:
+            #There is no existing convention from the GenBank SeqRecord parser
+            self.data.annotations['date_last_annotation_update'] = date_str
+
+    def keyword(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores a list as 'keywords'
+        cols = line[5:].rstrip(_CHOMP).split(';')
+        cols = [c.strip() for c in cols]
+        cols = filter(None, cols)
+        try :
+            #Extend any existing list of keywords
+            self.data.annotations['keywords'].extend(cols)
+        except KeyError :
+            #Create the list of keywords
+            self.data.annotations['keywords'] = cols
+
+    def organism_species(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores the organism as a string with key 'organism'
+        data = line[5:].rstrip(_CHOMP)
+        try :
+            #Append to any existing data split over multiple lines
+            self.data.annotations['organism'] += " " + data
+        except KeyError:
+            self.data.annotations['organism'] = data
+
+    def organism_host(self, line):
+        #There is no SeqRecord convention from the GenBank parser,
+        #based on how it deals with taxonomy ids (list of strings)
+        data = line[5:].rstrip(_CHOMP)
+        index = data.find('=')
+        if index >= 0:
+            descr = data[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = data[index+1:].split(',')
+        else:
+            ids = data.split(',')
+
+        try :
+            #Append to any existing data
+            self.data.annotations['organism_host'].extend(ids)
+        except KeyError:
+            self.data.annotations['organism_host'] = ids
+
+    def taxonomy_id(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores these as a list of strings with key 'taxonomy'
+
+        line = line[5:].rstrip(_CHOMP)
+        index = line.find('=')
+        if index >= 0:
+            descr = line[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = line[index+1:].split(',')
+        else:
+            ids = line.split(',')
+
+        try :
+            #Append to any existing data
+            self.data.annotations['taxonomy'].extend(ids)
+        except KeyError:
+            self.data.annotations['taxonomy'] = ids
+
 def index_file(filename, indexname, rec2key=None):
     """index_file(filename, indexname, rec2key=None)
 
@@ -922,3 +1144,32 @@ def index_file(filename, indexname, rec2key=None):
             raise KeyError, "duplicate key %s found" % key
 
         index[key] = start, length
+        
+if __name__ == "__main__" :
+    print "Quick self test..."
+
+    example_filename = "../../Tests/SwissProt/sp008"
+
+    import os
+    if not os.path.isfile(example_filename) :
+        print "Missing test file %s" % example_filename
+    else :
+        #Try parsing it!
+        
+        handle = open(example_filename)
+        for record in Iterator(handle, RecordParser()) :
+            print record.entry_name
+            print ",".join(record.accessions)
+            print record.keywords
+            print repr(record.organism)
+            print record.sequence[:20] + "..."
+        handle.close()
+
+        handle = open(example_filename)
+        for record in Iterator(handle, SequenceParser()) :
+            print record.name
+            print record.id
+            print record.annotations['keywords']
+            print repr(record.annotations['organism'])
+            print record.seq.tostring()[:20] + "..."
+        handle.close()
