@@ -1,8 +1,12 @@
 # Read a FASTA description
 
+import warnings
+warnings.warn("Bio.expressions was deprecated, as it does not work with recent versions of mxTextTools. If you want to continue to use this module, please get in contact with the Biopython developers at biopython-dev@biopython.org to avoid permanent removal of this module from Biopython", DeprecationWarning)
+
+
+
 import operator
 from Martel import *
-from Martel import RecordReader
 from Bio import Std
 
 ### Parse dbxrefs given the NCBI|descr|line as explained
