@@ -27,7 +27,7 @@ equal_weight    Every example is given a weight of 1.
 try:
     from Numeric import *
 except ImportError, x:
-    raise ImportError, "This module requires NumPy"
+    raise ImportError, "This module requires Numeric (precursor to NumPy)"
 
 from Bio import listfns
 from Bio import distance
